@@ -56,7 +56,7 @@ Examples of locations:
 * 'mod.Cls' represents each method in class 'mod.Cls' + the top-level of the
   module 'mod'. (To simplify the implementation, there is no location that only
   includes the body of a class without the entire surrounding module top level.)
-* Trigger '<...>' as a location is an indirect way of referring to to all
+* Trigger '<...>' as a location is an indirect way of referring to all
   locations triggered by the trigger. These indirect locations keep the
   dependency map smaller and easier to manage.
 
@@ -81,7 +81,7 @@ Test cases for this module live in 'test-data/unit/deps*.test'.
 
 from __future__ import annotations
 
-from typing import DefaultDict, List
+from collections import defaultdict
 
 from mypy.nodes import (
     GDEF,
@@ -171,7 +171,7 @@ from mypy.types import (
     UnpackType,
     get_proper_type,
 )
-from mypy.typestate import TypeState
+from mypy.typestate import type_state
 from mypy.util import correct_relative_import
 
 
@@ -220,7 +220,7 @@ class DependencyVisitor(TraverserVisitor):
         self,
         type_map: dict[Expression, Type],
         python_version: tuple[int, int],
-        alias_deps: DefaultDict[str, set[str]],
+        alias_deps: defaultdict[str, set[str]],
         options: Options | None = None,
     ) -> None:
         self.scope = Scope()
@@ -288,13 +288,9 @@ class DependencyVisitor(TraverserVisitor):
             # all call sites, making them all `Any`.
             for d in o.decorators:
                 tname: str | None = None
-                if isinstance(d, RefExpr) and d.fullname is not None:
+                if isinstance(d, RefExpr) and d.fullname:
                     tname = d.fullname
-                if (
-                    isinstance(d, CallExpr)
-                    and isinstance(d.callee, RefExpr)
-                    and d.callee.fullname is not None
-                ):
+                if isinstance(d, CallExpr) and isinstance(d.callee, RefExpr) and d.callee.fullname:
                     tname = d.callee.fullname
                 if tname is not None:
                     self.add_dependency(make_trigger(tname), make_trigger(o.func.fullname))
@@ -343,7 +339,7 @@ class DependencyVisitor(TraverserVisitor):
                 self.add_dependency(
                     make_wildcard_trigger(base_info.fullname), target=make_trigger(target)
                 )
-                # More protocol dependencies are collected in TypeState._snapshot_protocol_deps
+                # More protocol dependencies are collected in type_state._snapshot_protocol_deps
                 # after a full run or update is finished.
 
         self.add_type_alias_deps(self.scope.current_target())
@@ -475,7 +471,7 @@ class DependencyVisitor(TraverserVisitor):
                 self.add_dependency(make_trigger(class_name + ".__init__"))
                 self.add_dependency(make_trigger(class_name + ".__new__"))
             if isinstance(rvalue, IndexExpr) and isinstance(rvalue.analyzed, TypeAliasExpr):
-                self.add_type_dependencies(rvalue.analyzed.type)
+                self.add_type_dependencies(rvalue.analyzed.node.target)
             elif typ:
                 self.add_type_dependencies(typ)
         else:
@@ -499,7 +495,7 @@ class DependencyVisitor(TraverserVisitor):
             if (
                 isinstance(rvalue, CallExpr)
                 and isinstance(rvalue.callee, RefExpr)
-                and rvalue.callee.fullname is not None
+                and rvalue.callee.fullname
             ):
                 fname: str | None = None
                 if isinstance(rvalue.callee.node, TypeInfo):
@@ -509,7 +505,7 @@ class DependencyVisitor(TraverserVisitor):
                         fname = init.node.fullname
                 else:
                     fname = rvalue.callee.fullname
-                if fname is None:
+                if not fname:
                     return
                 for lv in o.lvalues:
                     if isinstance(lv, RefExpr) and lv.fullname and lv.is_new_def:
@@ -637,7 +633,7 @@ class DependencyVisitor(TraverserVisitor):
     # Expressions
 
     def process_global_ref_expr(self, o: RefExpr) -> None:
-        if o.fullname is not None:
+        if o.fullname:
             self.add_dependency(make_trigger(o.fullname))
 
         # If this is a reference to a type, generate a dependency to its
@@ -950,7 +946,7 @@ def get_type_triggers(
     return typ.accept(TypeTriggersVisitor(use_logical_deps, seen_aliases))
 
 
-class TypeTriggersVisitor(TypeVisitor[List[str]]):
+class TypeTriggersVisitor(TypeVisitor[list[str]]):
     def __init__(
         self, use_logical_deps: bool, seen_aliases: set[TypeAliasType] | None = None
     ) -> None:
@@ -968,6 +964,9 @@ class TypeTriggersVisitor(TypeVisitor[List[str]]):
             triggers.extend(self.get_type_triggers(arg))
         if typ.last_known_value:
             triggers.extend(self.get_type_triggers(typ.last_known_value))
+        if typ.extra_attrs and typ.extra_attrs.mod_name:
+            # Module as type effectively depends on all module attributes, use wildcard.
+            triggers.append(make_wildcard_trigger(typ.extra_attrs.mod_name))
         return triggers
 
     def visit_type_alias_type(self, typ: TypeAliasType) -> list[str]:
@@ -1027,7 +1026,7 @@ class TypeTriggersVisitor(TypeVisitor[List[str]]):
     def visit_type_type(self, typ: TypeType) -> list[str]:
         triggers = self.get_type_triggers(typ.item)
         if not self.use_logical_deps:
-            old_triggers = triggers[:]
+            old_triggers = triggers.copy()
             for trigger in old_triggers:
                 triggers.append(trigger.rstrip(">") + ".__init__>")
                 triggers.append(trigger.rstrip(">") + ".__new__>")
@@ -1039,6 +1038,8 @@ class TypeTriggersVisitor(TypeVisitor[List[str]]):
             triggers.append(make_trigger(typ.fullname))
         if typ.upper_bound:
             triggers.extend(self.get_type_triggers(typ.upper_bound))
+        if typ.default:
+            triggers.extend(self.get_type_triggers(typ.default))
         for val in typ.values:
             triggers.extend(self.get_type_triggers(val))
         return triggers
@@ -1047,6 +1048,10 @@ class TypeTriggersVisitor(TypeVisitor[List[str]]):
         triggers = []
         if typ.fullname:
             triggers.append(make_trigger(typ.fullname))
+        if typ.upper_bound:
+            triggers.extend(self.get_type_triggers(typ.upper_bound))
+        if typ.default:
+            triggers.extend(self.get_type_triggers(typ.default))
         triggers.extend(self.get_type_triggers(typ.upper_bound))
         return triggers
 
@@ -1054,6 +1059,10 @@ class TypeTriggersVisitor(TypeVisitor[List[str]]):
         triggers = []
         if typ.fullname:
             triggers.append(make_trigger(typ.fullname))
+        if typ.upper_bound:
+            triggers.extend(self.get_type_triggers(typ.upper_bound))
+        if typ.default:
+            triggers.extend(self.get_type_triggers(typ.default))
         triggers.extend(self.get_type_triggers(typ.upper_bound))
         return triggers
 
@@ -1119,7 +1128,7 @@ def dump_all_dependencies(
         deps = get_dependencies(node, type_map, python_version, options)
         for trigger, targets in deps.items():
             all_deps.setdefault(trigger, set()).update(targets)
-    TypeState.add_all_protocol_deps(all_deps)
+    type_state.add_all_protocol_deps(all_deps)
 
     for trigger, targets in sorted(all_deps.items(), key=lambda x: x[0]):
         print(trigger)
