@@ -8,7 +8,9 @@ import re
 import shutil
 import sys
 import time
-from typing import IO, Any, Callable, Iterable, Iterator, Pattern
+from collections.abc import Iterable, Iterator
+from re import Pattern
+from typing import IO, Any, Callable
 
 # Exporting Suite as alias to TestCase for backwards compatibility
 # TODO: avoid aliasing - import and subclass TestCase directly
@@ -41,7 +43,7 @@ def run_mypy(args: list[str]) -> None:
     if status != 0:
         sys.stdout.write(outval)
         sys.stderr.write(errval)
-        pytest.fail(msg="Sample check failed", pytrace=False)
+        pytest.fail(reason="Sample check failed", pytrace=False)
 
 
 def diff_ranges(
@@ -104,7 +106,9 @@ def render_diff_range(
             output.write("\n")
 
 
-def assert_string_arrays_equal(expected: list[str], actual: list[str], msg: str) -> None:
+def assert_string_arrays_equal(
+    expected: list[str], actual: list[str], msg: str, *, traceback: bool = False
+) -> None:
     """Assert that two string arrays are equal.
 
     Display any differences in a human-readable form.
@@ -133,10 +137,10 @@ def assert_string_arrays_equal(expected: list[str], actual: list[str], msg: str)
             show_align_message(expected[first_diff], actual[first_diff])
 
         sys.stderr.write(
-            "Update the test output using --update-data -n0 "
-            "(you can additionally use the -k selector to update only specific tests)\n"
+            "Update the test output using --update-data "
+            "(implies -n0; you can additionally use the -k selector to update only specific tests)\n"
         )
-        pytest.fail(msg, pytrace=False)
+        pytest.fail(msg, pytrace=traceback)
 
 
 def assert_module_equivalence(name: str, expected: Iterable[str], actual: Iterable[str]) -> None:
@@ -145,7 +149,7 @@ def assert_module_equivalence(name: str, expected: Iterable[str], actual: Iterab
     assert_string_arrays_equal(
         expected_normalized,
         actual_normalized,
-        ("Actual modules ({}) do not match expected modules ({}) " 'for "[{} ...]"').format(
+        ('Actual modules ({}) do not match expected modules ({}) for "[{} ...]"').format(
             ", ".join(actual_normalized), ", ".join(expected_normalized), name
         ),
     )
@@ -156,7 +160,7 @@ def assert_target_equivalence(name: str, expected: list[str], actual: list[str])
     assert_string_arrays_equal(
         expected,
         actual,
-        ("Actual targets ({}) do not match expected targets ({}) " 'for "[{} ...]"').format(
+        ('Actual targets ({}) do not match expected targets ({}) for "[{} ...]"').format(
             ", ".join(actual), ", ".join(expected), name
         ),
     )
@@ -254,16 +258,9 @@ def local_sys_path_set() -> Iterator[None]:
 
 
 def testfile_pyversion(path: str) -> tuple[int, int]:
-    if path.endswith("python312.test"):
-        return 3, 12
-    elif path.endswith("python311.test"):
-        return 3, 11
-    elif path.endswith("python310.test"):
-        return 3, 10
-    elif path.endswith("python39.test"):
-        return 3, 9
-    elif path.endswith("python38.test"):
-        return 3, 8
+    m = re.search(r"python3([0-9]+)\.test$", path)
+    if m:
+        return 3, int(m.group(1))
     else:
         return defaults.PYTHON3_VERSION
 
